from typing import ContextManager

from .repository import EventRepository


class EventRepositoryFactory(object):
    def build(self) -> ContextManager[EventRepository]:
        raise NotImplementedError()
