from abc import ABCMeta, abstractmethod

from messageprocessor import MessageProcessor


class MessageMessageProcessorBuilder(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def build_message_processor(
        self,
        token,  # type: int
    ):
        # type: (...) -> MessageProcessor
        raise NotImplementedError()
