from setuptools import setup, find_packages

setup(
    name="EventManagerApi",
    version="1.0.0",
    packages=find_packages(where="."),
    install_requires=[],
    url="https://www.e-deploy.com.br",
    author="Eduardo Almeida",
    author_email="eduardo.almeida@e-deploy.com.br"
)
