data = (
'jjael',    # 0x00
'jjaelg',    # 0x01
'jjaelm',    # 0x02
'jjaelb',    # 0x03
'jjaels',    # 0x04
'jjaelt',    # 0x05
'jjaelp',    # 0x06
'jjaelh',    # 0x07
'jjaem',    # 0x08
'jjaeb',    # 0x09
'jjaebs',    # 0x0a
'jjaes',    # 0x0b
'jjaess',    # 0x0c
'jjaeng',    # 0x0d
'jjaej',    # 0x0e
'jjaec',    # 0x0f
'jjaek',    # 0x10
'jjaet',    # 0x11
'jjaep',    # 0x12
'jjaeh',    # 0x13
'jjya',    # 0x14
'jjyag',    # 0x15
'jjyagg',    # 0x16
'jjyags',    # 0x17
'jjyan',    # 0x18
'jjyanj',    # 0x19
'jjyanh',    # 0x1a
'jjyad',    # 0x1b
'jjyal',    # 0x1c
'jjyalg',    # 0x1d
'jjyalm',    # 0x1e
'jjyalb',    # 0x1f
'jjyals',    # 0x20
'jjyalt',    # 0x21
'jjyalp',    # 0x22
'jjyalh',    # 0x23
'jjyam',    # 0x24
'jjyab',    # 0x25
'jjyabs',    # 0x26
'jjyas',    # 0x27
'jjyass',    # 0x28
'jjyang',    # 0x29
'jjyaj',    # 0x2a
'jjyac',    # 0x2b
'jjyak',    # 0x2c
'jjyat',    # 0x2d
'jjyap',    # 0x2e
'jjyah',    # 0x2f
'jjyae',    # 0x30
'jjyaeg',    # 0x31
'jjyaegg',    # 0x32
'jjyaegs',    # 0x33
'jjyaen',    # 0x34
'jjyaenj',    # 0x35
'jjyaenh',    # 0x36
'jjyaed',    # 0x37
'jjyael',    # 0x38
'jjyaelg',    # 0x39
'jjyaelm',    # 0x3a
'jjyaelb',    # 0x3b
'jjyaels',    # 0x3c
'jjyaelt',    # 0x3d
'jjyaelp',    # 0x3e
'jjyaelh',    # 0x3f
'jjyaem',    # 0x40
'jjyaeb',    # 0x41
'jjyaebs',    # 0x42
'jjyaes',    # 0x43
'jjyaess',    # 0x44
'jjyaeng',    # 0x45
'jjyaej',    # 0x46
'jjyaec',    # 0x47
'jjyaek',    # 0x48
'jjyaet',    # 0x49
'jjyaep',    # 0x4a
'jjyaeh',    # 0x4b
'jjeo',    # 0x4c
'jjeog',    # 0x4d
'jjeogg',    # 0x4e
'jjeogs',    # 0x4f
'jjeon',    # 0x50
'jjeonj',    # 0x51
'jjeonh',    # 0x52
'jjeod',    # 0x53
'jjeol',    # 0x54
'jjeolg',    # 0x55
'jjeolm',    # 0x56
'jjeolb',    # 0x57
'jjeols',    # 0x58
'jjeolt',    # 0x59
'jjeolp',    # 0x5a
'jjeolh',    # 0x5b
'jjeom',    # 0x5c
'jjeob',    # 0x5d
'jjeobs',    # 0x5e
'jjeos',    # 0x5f
'jjeoss',    # 0x60
'jjeong',    # 0x61
'jjeoj',    # 0x62
'jjeoc',    # 0x63
'jjeok',    # 0x64
'jjeot',    # 0x65
'jjeop',    # 0x66
'jjeoh',    # 0x67
'jje',    # 0x68
'jjeg',    # 0x69
'jjegg',    # 0x6a
'jjegs',    # 0x6b
'jjen',    # 0x6c
'jjenj',    # 0x6d
'jjenh',    # 0x6e
'jjed',    # 0x6f
'jjel',    # 0x70
'jjelg',    # 0x71
'jjelm',    # 0x72
'jjelb',    # 0x73
'jjels',    # 0x74
'jjelt',    # 0x75
'jjelp',    # 0x76
'jjelh',    # 0x77
'jjem',    # 0x78
'jjeb',    # 0x79
'jjebs',    # 0x7a
'jjes',    # 0x7b
'jjess',    # 0x7c
'jjeng',    # 0x7d
'jjej',    # 0x7e
'jjec',    # 0x7f
'jjek',    # 0x80
'jjet',    # 0x81
'jjep',    # 0x82
'jjeh',    # 0x83
'jjyeo',    # 0x84
'jjyeog',    # 0x85
'jjyeogg',    # 0x86
'jjyeogs',    # 0x87
'jjyeon',    # 0x88
'jjyeonj',    # 0x89
'jjyeonh',    # 0x8a
'jjyeod',    # 0x8b
'jjyeol',    # 0x8c
'jjyeolg',    # 0x8d
'jjyeolm',    # 0x8e
'jjyeolb',    # 0x8f
'jjyeols',    # 0x90
'jjyeolt',    # 0x91
'jjyeolp',    # 0x92
'jjyeolh',    # 0x93
'jjyeom',    # 0x94
'jjyeob',    # 0x95
'jjyeobs',    # 0x96
'jjyeos',    # 0x97
'jjyeoss',    # 0x98
'jjyeong',    # 0x99
'jjyeoj',    # 0x9a
'jjyeoc',    # 0x9b
'jjyeok',    # 0x9c
'jjyeot',    # 0x9d
'jjyeop',    # 0x9e
'jjyeoh',    # 0x9f
'jjye',    # 0xa0
'jjyeg',    # 0xa1
'jjyegg',    # 0xa2
'jjyegs',    # 0xa3
'jjyen',    # 0xa4
'jjyenj',    # 0xa5
'jjyenh',    # 0xa6
'jjyed',    # 0xa7
'jjyel',    # 0xa8
'jjyelg',    # 0xa9
'jjyelm',    # 0xaa
'jjyelb',    # 0xab
'jjyels',    # 0xac
'jjyelt',    # 0xad
'jjyelp',    # 0xae
'jjyelh',    # 0xaf
'jjyem',    # 0xb0
'jjyeb',    # 0xb1
'jjyebs',    # 0xb2
'jjyes',    # 0xb3
'jjyess',    # 0xb4
'jjyeng',    # 0xb5
'jjyej',    # 0xb6
'jjyec',    # 0xb7
'jjyek',    # 0xb8
'jjyet',    # 0xb9
'jjyep',    # 0xba
'jjyeh',    # 0xbb
'jjo',    # 0xbc
'jjog',    # 0xbd
'jjogg',    # 0xbe
'jjogs',    # 0xbf
'jjon',    # 0xc0
'jjonj',    # 0xc1
'jjonh',    # 0xc2
'jjod',    # 0xc3
'jjol',    # 0xc4
'jjolg',    # 0xc5
'jjolm',    # 0xc6
'jjolb',    # 0xc7
'jjols',    # 0xc8
'jjolt',    # 0xc9
'jjolp',    # 0xca
'jjolh',    # 0xcb
'jjom',    # 0xcc
'jjob',    # 0xcd
'jjobs',    # 0xce
'jjos',    # 0xcf
'jjoss',    # 0xd0
'jjong',    # 0xd1
'jjoj',    # 0xd2
'jjoc',    # 0xd3
'jjok',    # 0xd4
'jjot',    # 0xd5
'jjop',    # 0xd6
'jjoh',    # 0xd7
'jjwa',    # 0xd8
'jjwag',    # 0xd9
'jjwagg',    # 0xda
'jjwags',    # 0xdb
'jjwan',    # 0xdc
'jjwanj',    # 0xdd
'jjwanh',    # 0xde
'jjwad',    # 0xdf
'jjwal',    # 0xe0
'jjwalg',    # 0xe1
'jjwalm',    # 0xe2
'jjwalb',    # 0xe3
'jjwals',    # 0xe4
'jjwalt',    # 0xe5
'jjwalp',    # 0xe6
'jjwalh',    # 0xe7
'jjwam',    # 0xe8
'jjwab',    # 0xe9
'jjwabs',    # 0xea
'jjwas',    # 0xeb
'jjwass',    # 0xec
'jjwang',    # 0xed
'jjwaj',    # 0xee
'jjwac',    # 0xef
'jjwak',    # 0xf0
'jjwat',    # 0xf1
'jjwap',    # 0xf2
'jjwah',    # 0xf3
'jjwae',    # 0xf4
'jjwaeg',    # 0xf5
'jjwaegg',    # 0xf6
'jjwaegs',    # 0xf7
'jjwaen',    # 0xf8
'jjwaenj',    # 0xf9
'jjwaenh',    # 0xfa
'jjwaed',    # 0xfb
'jjwael',    # 0xfc
'jjwaelg',    # 0xfd
'jjwaelm',    # 0xfe
'jjwaelb',    # 0xff
)
