from abc import abstractmethod
from typing import Dict


class AsyncTargetSubjectIdFieldMapRepository(object):
    @abstractmethod
    def update_map(self, subject: str, id_field: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def load_map(self) -> Dict[str, str]:
        raise NotImplementedError()
