class CloseOperator(object):
    def __init__(self, pos_id, values, envelope, session, payments, manager_id, period):
        self.pos_id = pos_id
        self.values = values
        self.envelope = envelope
        self.session = session
        self.payments = payments
        self.manager_id = manager_id
        self.period = period
